package org.china.vo.das.activity;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.util.Map;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import uk.org.ogsadai.activity.Activity;
import uk.org.ogsadai.activity.ActivityCreationException;
import uk.org.ogsadai.activity.ActivityExecutionException;
import uk.org.ogsadai.activity.ActivitySpecificationException;
import uk.org.ogsadai.activity.ActivityStreamNotFoundException;
import uk.org.ogsadai.common.Base64Reader;
import uk.org.ogsadai.common.BlockReader;
import uk.org.ogsadai.common.BlockWriter;
import uk.org.ogsadai.common.UnsupportedBlockException;
import uk.org.ogsadai.converters.resultset.ResultSetConverter;
import uk.org.ogsadai.converters.resultset.ResultSetHandlerException;
import uk.org.ogsadai.engine.EngineConstants;
import uk.org.ogsadai.exception.DAIIOException;
import uk.org.ogsadai.service.OGSADAIConstants;
import org.china.vo.das.converters.resultset.*;


/**
 * Implementation of SQL <code>java.sql.ResultSet</code> to WebRowSet
 * conversion activity (<code>sqlResultstoXML</code>).
 * <p>
 * For information on this activity see the OGSA-DAI user doc:
 * <code>OGSA-DAI/doc/interaction/activities/relational/sqlResultsToXML.html
 * </code>
 * </p>
 *
 * @author The OGSA-DAI Project Team.
 */
public class SQLResult2VOTableActivity extends Activity 
{
	   /** Copyright statement */
    private static final String COPYRIGHT_NOTICE =
        "(c) University of Edinburgh 2002 - 2006";

    /** Constants for parsing activity XML */
    public static final String RESULT_SET_ELEMENT = "resultSet";
    public static final String VOTABLE_ELEMENT = "voTable";
    public static final String FROM_ATTRIBUTE = "from";
    public static final String NAME_ATTRIBUTE = "name";
    public static final String NULL_DATA_TAG = "<null/>";
    
    /** Activity input name **/
    private String mInputName;
    /** Activity input - <code>java.sql.ResultSet</code> **/
    private BlockReader mInput;
    
    /** Activity output name **/
    private String mOutputName;
    /** Activity output - WebRowSet XML as <code>java.lang.Strings</code>  **/
    private BlockWriter mOutput;
    
    /** <code>java.sql.ResultSet</code> to WebRowSet converter **/
    private ResultSetConverter mVOTableConvertor;
    
    /**
     * Constructs an activity using the specified element.
     * 
     * @param element
     *            <code>Element</code> to configure the activity with - from a
     *            perform document.
     * @throws ActivityCreationException
     *             If there is a problem constructing the activity.
     * @throws ActivitySpecificationException
     *             If there is a problem constructing the activity due to an
     *             invalid setting in <code>element</code>.
     * 
     * @see uk.org.ogsadai.activity#Activity(Element)
     */
    public SQLResult2VOTableActivity(Element element) 
        throws ActivitySpecificationException, ActivityCreationException 
    {
        super(element);
        NodeList nodelist = element.getElementsByTagNameNS(
            OGSADAIConstants.TYPES_NAMESPACE, RESULT_SET_ELEMENT);
        if ((nodelist == null) || (nodelist.getLength() == 0)) 
        {
            throw new ActivitySpecificationException(getName(), RESULT_SET_ELEMENT);
        }
        String inputName = ((Element)nodelist.item(0)).getAttribute(
            FROM_ATTRIBUTE);
        mInternalInputs = new String[] {inputName};
        nodelist = element.getElementsByTagNameNS(
            OGSADAIConstants.TYPES_NAMESPACE, VOTABLE_ELEMENT);
        if ((nodelist == null) || (nodelist.getLength() == 0)) 
        {
            throw new ActivitySpecificationException(getName(), VOTABLE_ELEMENT);
        }
        String outputName = ((Element)nodelist.item(0)).getAttribute(
            NAME_ATTRIBUTE);
        mInternalOutputs = new String[] {outputName};
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.activity.Activity#initialise()
     */
    public void initialise()
        throws ActivitySpecificationException, ActivityExecutionException 
    {
        super.initialise();
        try
        {
            mInput = mContext.getInput( mInternalInputs[0] );
            mOutput = mContext.getOutput( mInternalOutputs[0] );
        }
        catch (ActivityStreamNotFoundException e) 
        {
            throw new ActivitySpecificationException(getName(), e);
        }
    }
    
    /* (non-Javadoc)
     * @see uk.org.ogsadai.engine.Activity#processBlock()
     */
    protected void processBlock() 
    {
        try
        {
            if ((mVOTableConvertor != null) && mVOTableConvertor.hasNext())
            {
                String rowsetBlock = new String(mVOTableConvertor.next());
                mOutput.put(rowsetBlock);
            }
            else
            {
            	mVOTableConvertor = null;
                if (mInput.hasNext()) 
                {
                    Object block = mInput.next();
                    if (block.equals(EngineConstants.NO_MORE_DATA)) 
                    {
                        setCompleted();
                    }
                    else if (block instanceof String) 
                    {
                        // We have received an update count
                        // or some other control block
                        mOutput.put(block);
                    }
                    else if (block instanceof Map) 
                    {
                        // We have received a map containing output
                        // parameters position |-> value
                        String xml = getOutParametersAsXML((Map)block);
                        mOutput.put(xml);
                    }
                    else 
                    {
                        ResultSet resultSet = (ResultSet)block;
                        mVOTableConvertor = new ResultSetConverter(resultSet, 
                            new VOTableResultSetHandler(resultSet));
                    }
                }        
                else 
                {
                    setCompleted();
                }
            }
        } 
        catch (ResultSetHandlerException e) 
        {
            setError(new ActivityExecutionException(getName(), e));
        } 
        catch (SQLException e) 
        {
            setError(new ActivityExecutionException(getName(), e));
        } 
        catch (IOException e) 
        {
            setError(new ActivityExecutionException(getName(), 
                                                    new DAIIOException(e)));
        }
    }
    
    /**
     * Given a mapping from <code>java.lang.Integer</code> to
     * <code>java.lang.Object</code> return an XML representation of
     * the mapping.
     *
     * @param params
     *     The mapping to convert.
     * @return output parameters as XML
     * @throws SQLException
     *     If problems arise when converting <code>java.sql</code>
     *     objects.
     * @throws IOException
     *     If problems arise converting <code>java.sql.Blob</code>
     *     or <code>java.sql.Clob</code> objects.
     *
     * @see #getAsString(java.lang.Object)
     */
    private String getOutParametersAsXML(Map params) 
        throws SQLException, 
               IOException 
    {
        StringBuffer sb = new StringBuffer();
        Integer[] positions = (Integer[]) params.keySet().toArray(
            new Integer[params.size()]);
        int numPositions = positions.length;
        for (int i = 0; i < numPositions; i++) 
        {
            sb.append("<outputParameter position=\"");
            sb.append(positions[i]);
            sb.append("\">");
            Object obj = params.get(positions[i]);
            sb.append(getAsString(obj));
            sb.append("</outputParameter>\n");
        }        
        return sb.toString();
    }
    
    /**
     * Gets an object as a string. The method handles
     * <code>null</code>, <code>java.sql.Blob</code> and
     * <code>java.sql.Clob</code>, <code>java.sql.Date</code> and
     * <code>java.sql.Time</code> as special cases.
     *
     * @param obj
     *    Object to convert.
     * @return string representation
     * @throws SQLException
     *     If problems arise when converting <code>java.sql</code>
     *     objects.
     * @throws IOException
     *     If problems arise converting <code>java.sql.Blob</code>
     *     or <code>java.sql.Clob</code> objects.
     *
     * @see #replaceSpecialCharacters(java.lang.String)
     */
    private String getAsString(Object obj) 
        throws SQLException,
               IOException 
    {
        String result = "";
        if (obj == null) 
        {
            result = NULL_DATA_TAG;
        }
        else if (obj instanceof Blob) 
        {
            Blob blob = (Blob)obj;
            InputStream binaryStream = blob.getBinaryStream();
            if (binaryStream != null)
            {
                Base64Reader reader = new Base64Reader(binaryStream);
                String encodedString = reader.readAll();
                reader.close();
                if (encodedString.length() > 0)
                {
                    result = encodedString;
                }
                encodedString = null;
                reader = null;
                binaryStream = null;
            }
        }
        else if (obj instanceof Clob) 
        {
            Reader reader = ((Clob)obj).getCharacterStream();
            if (reader != null) 
            {
                StringBuffer sb = new StringBuffer();
                char[] buf = new char[4096];
                while (reader.read(buf) > -1) 
                {
                    sb.append(buf);
                }
                result = sb.toString();
            }
        }
        else if (obj instanceof Time) 
        {
            result = Long.toString(((Time)obj).getTime());
        }
        else if (obj instanceof Date) 
        {
            result = Long.toString(((Date)obj).getTime());
        }
        else 
        {
            result = replaceSpecialCharacters(obj.toString());
        }
        return result;
    }

    /**
     * Replaces the special characters in the given input with the 
     * equivalent predefined entities.
     * 
     * @param input 
     *     String to process
     * @return the input string, with special characters replaced by the
     *         equivalent predefined entities.
     */
    protected String replaceSpecialCharacters(String input)
    {
        input = input.replaceAll("&", "&amp;");
        input = input.replaceAll(">",  "&gt;");
        input = input.replaceAll("<",  "&lt;");
        input = input.replaceAll("\"", "&quot;");
        input = input.replaceAll("\'", "&apos;");
        return input;
    }   
}
