package org.china.vo.das.activity.client;

import java.sql.ResultSet;

import uk.org.ogsadai.client.toolkit.activity.Activity;
import uk.org.ogsadai.client.toolkit.activity.ActivityOutput;
import uk.org.ogsadai.client.toolkit.activity.NoActivityOutputException;
import uk.org.ogsadai.client.toolkit.activity.sql.ResultSetActivityOutputConverter;
import uk.org.ogsadai.client.toolkit.exception.DataFormatException;

/**
 * This activity converts the output of SQL activities that produce
 * <code>java.sql.ResultSet</code> objects into VOTable format.
 * <p>
 * The activity has one input - the <code>ResultSet</code> - and one
 * output - the VOTable format.
 *
 * @author The VO-DAS Project Team
 */
public class SQLResult2VOTableClient extends Activity{
	
	/** Copyright statement */
    private static final String COPYRIGHT_NOTICE =
        "(c) NAOC of CAS 2002 - 2006";

    /** Type of <code>ResultSet</code>s generated using this class */
    private int mResultSetType;
    
    /**
     * Convert the <code>ResultSet</code> from the given activity
     * output into VOTable.
     * 
     * @param input
     *     Output from another activity.
     * @throws IllegalArgumentException
     *     If <code>input</code> is <code>null</code>.
     */
    public SQLResult2VOTableClient(ActivityOutput input)
    {
        addInput("ResultSet");
        addOutput("VOTable");
        setInput(input);
        mResultSetType = ResultSet.TYPE_FORWARD_ONLY;
    }
    
    /**
     * Sets the input of this activity to be the output from another
     * activity that will provide the <code>ResultSet</code> data.
     *
     * @param input
     *     Output from another activity.
     * @throws IllegalArgumentException
     *     If <code>input</code> is <code>null</code>.
     */
    public final void setInput(final ActivityOutput input) 
    {
        if (input == null) 
        {
            throw new IllegalArgumentException("input must not be null");
        }
        super.setInput(0, input.getName());
    }

    /**
     * Return the type of <code>ResultSet</code> 
     * objects the <code>getResultSet</code> method returns.
     * 
     * @return the <code>ResultSet</code> type
     */
    public int getResultSetType() 
    {
        return mResultSetType;
    }

    /**
     * Set the type of <code>ResultSet</code> objects the
     * <code>getResultSet</code> method returns.
     * 
     * @param resultSetType
     *      Either of <code>ResultSet.TYPE_FORWARD_ONLY</code> or
     *      <code>ResultSet.TYPE_SCROLL_INSENSITIVE</code>.
     * @throws IllegalArgumentException 
     *     if <code>resultSetType</code> is not one of the two above.
     */
    public void setResultSetType(int resultSetType) 
    {
        if ((resultSetType != ResultSet.TYPE_SCROLL_INSENSITIVE) &&
            (resultSetType != ResultSet.TYPE_FORWARD_ONLY)) 
        {
            throw new IllegalArgumentException("type must be one of ResultSet.TYPE_SCROLL_INSENSITIVE or ResultSet.TYPE_FORWARD_ONLY");
        }
        mResultSetType = resultSetType;
    }

    /**
     * Get the WebRowSet XML output from this activity and convert it
     * into a <code>ResultSet</code> of type specified by
     * <code>getResultSetType</code>.
     * 
     * @return <code>ResultSet</code> containing query result.
     * @throws NoActivityOutputException
     *     if there is no output data or the data has not yet been
     *     processed to this output or the activity that generates
     *     this output has not yet been executed.
     * @throws DataFormatException
     *     if the output from the activity cannot be parsed into a
     *     <code>ResultSet</code>.
     */
    public ResultSet getResultSet()
        throws NoActivityOutputException,
               DataFormatException
    {
        ActivityOutput resultSetOutput = getOutput();
        return ResultSetActivityOutputConverter.convert(resultSetOutput, 
                                                        mResultSetType);
    }

    /**
     * Gets the activity's only output - the XML WebRowSet.
     * 
     * @return the activity output.
     */    
    public ActivityOutput getOutput()
    {
        return getOutputs()[0];
    }
    
    /* (non-Javadoc)
     * @see uk.org.ogsadai.client.toolkit.Activity#generateXML()
     */
    protected String generateXML()
    {
        StringBuffer sb = new StringBuffer();
        sb.append("<sqlResultsToVOTable name=\"");
        sb.append(getName());
        sb.append("\">\n");
        sb.append("<resultSet from=\"");
        sb.append(replaceSpecialCharacters(getInputParameters()[0].getOutputName()));
        sb.append("\"/>\n");
        sb.append("<voTable name=\"");
        sb.append(getOutput().getName());
        sb.append("\"/>\n");
        sb.append("</sqlResultsToVOTable>\n");
        return sb.toString();
    }
}