// (c) University of Edinburgh, 2005.
// See OGSA-DAI-Licence.txt for licencing information.

package org.china.vo.das.converters.resultset;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.Statement;
import java.sql.Types;

import uk.org.ogsadai.converters.resultset.ResultSetHandler;
import uk.org.ogsadai.converters.resultset.ResultSetHandlerException;
import uk.org.ogsadai.converters.resultset.StrategicResultSetHandler;

/**
 * A <code>ResultSetHandler</code> that uses the
 * <em>strategy</em> design pattern and builds an XML WebRowSet
 * representation of a <code>java.sql.ResultSet</code>.
 * <p>
 * Key columns and maps are not supported at present.
 *
 * @author The OGSA-DAI Project Team.
 */
public class VOTableResultSetHandler extends StrategicResultSetHandler
    implements ResultSetHandler
{
    // Copyright statement
    private static final String COPYRIGHT_NOTICE =
        "(c) University of Edinburgh 2002 - 2005.";

    /**
     * Constructs a new handler.
     * @param rs
     *     Result set to convert
     * @throws ResultSetHandlerException
     *     If a problem occurs when accessing the ResultSet.
     */
    public VOTableResultSetHandler(ResultSet rs) 
        throws ResultSetHandlerException 
    {
        super(rs, new VOTableColumnStrategyFactory());
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.converters.resultset.ResultSetHandler#header(java.lang.StringBuffer, java.sql.ResultSet)
     */
    public void header(StringBuffer output, ResultSet rs)
        throws ResultSetHandlerException
    {        
        try
        {
            StringBuffer header = new StringBuffer();
            header.append("<?xml version=\"1.0\"?>\n ");
//            header.append("<!DOCTYPE VOTABLE SYSTEM \"http://us-vo.org/xml/VOTable.dtd\">\n");
            header.append("<VOTABLE version=\"1.1\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"http://www.ivoa.net/xml/VOTable/VOTable/v1.1\">\n");
            header.append(" <COOSYS ID=\"J2000\" equinox=\"J2000.\" epoch=\"J2000.\" system=\"eq_FK5\"/> \n");
           // header.append(" <RESOURCE name = \"" + metaData ">\n");
//            header.append(properties(rs));
            header.append(metadata(rs.getMetaData()));
            header.append("\n<DATA>\n<TABLEDATA>\n");
            output.append(header);
        }
        catch (Exception e)
        {
            throw new ResultSetHandlerException(e);
        }
    }
    
    // XML tags delimiting rows and columns 
    private final String ROWSTART_TAG = "<TR>\n";
    private final String ROWEND_TAG   = "</TR>";
    private final String COLSTART_TAG = "<TD>";
    private final String COLEND_TAG   = "</TD>\n";    
    
    /* (non-Javadoc)
     * @see uk.org.ogsadai.converters.resultset.ResultSetHandler#rowStart(java.lang.StringBuffer)
     */
    public void rowStart(StringBuffer output)
    {
        output.append(ROWSTART_TAG);
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.converters.resultset.ResultSetHandler#columnStart(java.lang.StringBuffer)
     */
    public void columnStart(StringBuffer output) 
    {
        output.append(COLSTART_TAG);
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.converters.resultset.ResultSetHandler#columnEnd(java.lang.StringBuffer)
     */
    public void columnEnd(StringBuffer output)
    {
        output.append(COLEND_TAG);
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.converters.resultset.ResultSetHandler#rowEnd(java.lang.StringBuffer)
     */
    public void rowEnd(StringBuffer output)
    {
        output.append(ROWEND_TAG);
    }

    /* (non-Javadoc)
     * @see uk.org.ogsadai.converters.resultset.ResultSetHandler#footer(java.lang.StringBuffer)
     */
    public void footer(StringBuffer output)
    {
        output.append("</TABLEDATA>\n</DATA>\n</TABLE>\n</RESOURCE>\n</VOTABLE>");
    }
    
    /**
     * Writes the <code>properties</code> element of a WebRowSet document.
     *
     * @param rs
     *     Result set.
     * @return element as a string.
     * @throws ResultSetHandlerException
     *     If a problem occurs when accessing the ResultSet.
     */
    private String properties(ResultSet rs)
        throws ResultSetHandlerException
    {
        try
        {
            StringBuffer properties = new StringBuffer("<properties>\n");
            properties.append("<command></command>\n");
            properties.append("<concurrency>");
            properties.append(rs.getConcurrency());
            properties.append("</concurrency>\n");
            properties.append("<datasource></datasource>\n");
            properties.append("<escape-processing>true</escape-processing>\n");
            String property; 
            try 
            {
                property = String.valueOf(rs.getFetchDirection());
            }
            catch (Exception e) 
            {
                property = "";
            }
            properties.append("<fetch-direction>");
            properties.append(property);
            properties.append("</fetch-direction>\n");
            try 
            {
                property = String.valueOf(rs.getFetchSize());
            }
            catch (Exception e) 
            {
                property = "";
            }
            properties.append("<fetch-size>");
            properties.append(property);
            properties.append("</fetch-size>\n");
            properties.append("<isolation-level>0</isolation-level>\n");
            // Not supported at present.
            properties.append("<key-columns></key-columns>\n");
            // Not supported at present.
            properties.append("<map></map>\n");
            Statement statement = rs.getStatement();
            try 
            {
                property = String.valueOf(statement.getMaxFieldSize());
            }
            catch (Exception e) 
            {
                property = "";
            }
            properties.append("<max-field-size>");
            properties.append(property);
            properties.append("</max-field-size>\n");
            try 
            {
                property = String.valueOf(statement.getMaxRows());
            }
            catch (Exception e) 
            {
                property = "";
            }
            properties.append("<max-rows>");
            properties.append(property);
            properties.append("</max-rows>\n");
            try 
            {
                property = String.valueOf(statement.getQueryTimeout());
            }
            catch (Exception e) 
            {
                property = "";
            }
            properties.append("<query-timeout>");
            properties.append(property);
            properties.append("</query-timeout>\n");
            properties.append("<read-only>true</read-only>\n");
            int rowsetType;
            try 
            {
                rowsetType = rs.getType();
            }
            catch (Exception e) 
            {
                rowsetType = -1;
            }
            String sType = "";
            switch (rowsetType) 
            {
                case ResultSet.TYPE_FORWARD_ONLY:
                    sType = "ResultSet.TYPE_FORWARD_ONLY";
                    break;
                case ResultSet.TYPE_SCROLL_INSENSITIVE:
                    sType = "ResultSet.TYPE_SCROLL_INSENSITIVE";
                    break;
                case ResultSet.TYPE_SCROLL_SENSITIVE:
                    sType = "ResultSet.TYPE_SCROLL_SENSITIVE";
                    break;
            }
            properties.append("<rowset-type>");
            properties.append(sType);
            properties.append("</rowset-type>\n");
            properties.append("<show-deleted>false</show-deleted>\n");
            properties.append("<table-name></table-name>\n");
            properties.append("<url></url>\n");
            properties.append("<sync-provider>\n");
            properties.append("<sync-provider-name/>\n");
            properties.append("<sync-provider-vendor/>\n");
            properties.append("<sync-provider-version/>\n");
            properties.append("<sync-provider-grade/>\n");
            properties.append("<data-source-lock/>\n");
            properties.append("</sync-provider>\n");
            properties.append("</properties>\n");
            return properties.toString();
       }
       catch (Exception e)
       {
           throw new ResultSetHandlerException(e);
       }
    }
    
    /**
     * Writes the <code>metadata</code> element of a WebRowSet document.
     *
     * @param metaData 
     *     ResultSet meta data.
     * @return element as a string.
     * @throws ResultSetHandlerException
     *     If a problem occurs when accessing the ResultSet.
     */
    private String metadata(ResultSetMetaData metaData) 
        throws ResultSetHandlerException
    {
        try 
        {
            StringBuffer metadata = new StringBuffer(" <RESOURCE name=\"" + metaData.getCatalogName(1)+ "\">\n");
            metadata.append("<TABLE name=\"" + metaData.getTableName(1) + "\">\n");
            metadata.append(" <DESCRIPTION>The query service is from China-VO</DESCRIPTION>\n");
            metadata.append(" <PARAM name=\"\" datatype=\"\" ucd=\"phys.size;instr.tel\" unit=\"\" value=\"\"/>\n");
            int noOfCols = metaData.getColumnCount();
//            metadata.append("<column-count>" + Integer.toString(noOfCols) + 
//                            "</column-count>\n");
            for(int i = 1; i <= noOfCols; i++)
            {
                metadata.append("<FIELD ");
                metadata.append("name=\"");
                metadata.append(metaData.getColumnName(i));
                metadata.append("\" label=\"");
                metadata.append(metaData.getColumnLabel(i));
                metadata.append("\" ID=\"");
                metadata.append("col" + i);
                if(metaData.getColumnLabel(i).startsWith("spfileurl")){
                	metadata.append("\" ucd=\"");
                    metadata.append("meta.ref.uri.spec");
                }else{
                	metadata.append("\" ucd=\"");
                    metadata.append("");
                }
                metadata.append("\" ref=\"");
                metadata.append("J2000");
                metadata.append("\" datatype=\"");
                metadata.append(metaData.getColumnTypeName(i));
                metadata.append("\" width=\"");             
                metadata.append(metaData.getScale(i));
                metadata.append("\" precision=\"");
                // BLOB columns do not have a numeric precision and some JDBC 
                // drivers raise exceptions when the getPrecision method is
                // invoked for such a column, so a default value of 0 is used. 
                if (metaData.getColumnType(i) == Types.BLOB)
                {
                    metadata.append("0");
                }
                else
                {
                    metadata.append(metaData.getPrecision(i));
                }
                metadata.append("\" unit=\"");
                metadata.append("");
                metadata.append("\" arraysize=\"");
                metadata.append("");
                metadata.append("\">\n");
                metadata.append("<DESCRIPTION>");
                metadata.append(metaData.getSchemaName(i));
                metadata.append("</DESCRIPTION>\n");
                metadata.append("</FIELD>\n");
            }
//            metadata.append("</TABLE>\n");
            return metadata.toString();
        }
        catch (Exception e)
        {
            throw new ResultSetHandlerException(e);
        }
    }
}
